﻿///////////////////////////////////////////////////////////////////////////
//  Copyright (C) Wizardry and Steamworks 2016 - License: GNU GPLv3      //
//  Please see: http://www.gnu.org/licenses/gpl.html for legal details,  //
//  rights of fair usage, the disclaimer and warranty conditions.        //
///////////////////////////////////////////////////////////////////////////

using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Threading.Tasks;

namespace Winify.Utilities
{
    public static class AES
    {
        #region Static Fields and Constants

        private const int AESKeySaltBytes = 16;

        private static readonly RNGCryptoServiceProvider Rng = new RNGCryptoServiceProvider();

        #endregion

        #region Public Methods

        ///////////////////////////////////////////////////////////////////////////
        //  Copyright (C) Wizardry and Steamworks 2016 - License: GNU GPLv3      //
        ///////////////////////////////////////////////////////////////////////////
        public static async Task<byte[]> Encrypt(byte[] plain, string password)
        {
            using var rijndael = Rijndael.Create();
            rijndael.BlockSize = 128;
            rijndael.Mode = CipherMode.CBC;
            rijndael.KeySize = 256;
            rijndael.Padding = PaddingMode.PKCS7;
            var salt = new byte[AESKeySaltBytes];
            Rng.GetBytes(salt);
            using var pdb = new Rfc2898DeriveBytes(password, salt, 1000);
            rijndael.Key = pdb.GetBytes(rijndael.KeySize / 8);
            rijndael.IV = pdb.GetBytes(rijndael.BlockSize / 8);

            using var memoryStream = new MemoryStream();
            using var cryptoStream =
                new CryptoStream(memoryStream, rijndael.CreateEncryptor(), CryptoStreamMode.Write);
            await cryptoStream.WriteAsync(plain, 0, plain.Length);
            cryptoStream.Close();

            var ciphertext = memoryStream.ToArray();

            return salt.Concat(ciphertext).ToArray();
        }

        ///////////////////////////////////////////////////////////////////////////
        //  Copyright (C) Wizardry and Steamworks 2016 - License: GNU GPLv3      //
        ///////////////////////////////////////////////////////////////////////////
        public static async Task<byte[]> Decrypt(byte[] cipher, string password)
        {
            using var rijndael = Rijndael.Create();
            rijndael.BlockSize = 128;
            rijndael.Mode = CipherMode.CBC;
            rijndael.KeySize = 256;
            rijndael.Padding = PaddingMode.PKCS7;

            var salt = cipher.Take(AESKeySaltBytes).ToArray();

            var ciphertext = cipher.Skip(AESKeySaltBytes).ToArray();

            using var pdb = new Rfc2898DeriveBytes(password, salt, 1000);
            rijndael.Key = pdb.GetBytes(rijndael.KeySize / 8);
            rijndael.IV = pdb.GetBytes(rijndael.BlockSize / 8);

            using var memoryStream = new MemoryStream();
            using var cryptoStream =
                new CryptoStream(memoryStream, rijndael.CreateDecryptor(), CryptoStreamMode.Write);
            await cryptoStream.WriteAsync(ciphertext, 0, ciphertext.Length);
            cryptoStream.Close();

            return memoryStream.ToArray();
        }

        #endregion
    }
}